@@ -38,13 +38,14 @@ def target_drag_leave(w, context, time):
     global have_drag
     print 'leave'
     have_drag = gtk.FALSE
-    w.set(trashcan_closed, trashcan_closed_mask)
+    w.set_from_pixmap(trashcan_closed, trashcan_closed_mask)
+    
 def target_drag_motion(w, context, x, y, time):
     global trashcan_open, trashcan_open_mask
     global have_drag
     if not have_drag:
 	have_drag = gtk.TRUE
-	w.set(trashcan_open, trashcan_open_mask)
+	w.set_from_pixmap(trashcan_open, trashcan_open_mask)
     source_widget = context.get_source_widget()
     print 'motion, source ',
     if source_widget:
@@ -53,28 +54,32 @@ def target_drag_motion(w, context, x, y, time):
 	print 'unknown'
     context.drag_status(context.suggested_action, time)
     return gtk.TRUE
+
 def target_drag_drop(w, context, x, y, time):
     global trashcan_closed, trashcan_closed_mask
     global have_drag
     print 'drop'
     have_drag = gtk.FALSE
-    w.set(trashcan_closed, trashcan_closed_mask)
+    w.set_from_pixmap(trashcan_closed, trashcan_closed_mask)
     if context.targets:
 	w.drag_get_data(context, context.targets[0], time)
 	return gtk.TRUE
     return gtk.FALSE
+
 def target_drag_data_received(w, context, x, y, data, info, time):
     if data.format == 8:
 	print 'Received "%s" in trashcan' % data.data
 	context.finish(gtk.TRUE, gtk.FALSE, time)
     else:
 	context.finish(gtk.FALSE, gtk.FALSE, time)
+        
 def label_drag_data_received(w, context, x, y, data, info, time):
     if data and data.format == 8:
 	print 'Received "%s" in label' % data.data
 	context.finish(gtk.TRUE, gtk.FALSE, time)
     else:
 	context.finish(gtk.FALSE, gtk.FALSE, time)
+        
 def source_drag_data_get(w, context, selection_data, info, time):
     if info == TARGET_ROOTWIN:
 	print 'I was dropped on the rootwin'
@@ -88,6 +93,7 @@ def popdown_cb():
     popup_win.hide()
     popped_up = gtk.FALSE
     return gtk.FALSE
+
 def popup_motion(w, context, x, y, time):
     global in_popup, popdown_timer
     if not in_popup:
@@ -97,6 +103,7 @@ def popup_motion(w, context, x, y, time):
 	    gtk.timeout_remove(popdown_timer)
 	    popdown_timer = 0
     return gtk.TRUE
+
 def popup_leave(w, context, time):
     global in_popup, popdown_timer
     print 'popup_leave'
@@ -105,6 +112,7 @@ def popup_leave(w, context, time):
 	if not popdown_timer:
 	    print 'added popdown'
 	    popdown_timer = gtk.timeout_add(500, popdown_cb)
+            
 def popup_cb():
     global popped_up, popup_win
     global popup_timer, popdown_timer
@@ -129,18 +137,22 @@ def popup_cb():
     print 'added popdown'
     popup_timer = 0
     return gtk.FALSE
+
 def popsite_motion(w, context, x, y, time):
     global popup_timer
     if not popup_timer:
 	popup_timer = gtk.timeout_add(500, popup_cb)
     return gtk.TRUE
+
 def popsite_leave(w, context, time):
     global popup_timer
     if popup_timer:
 	gtk.timeout_remove(popup_timer)
 	popup_timer = 0
+        
 def source_drag_data_delete(w, context, data):
     print 'Delete the data!'
+    
 def create_pixmap(widget, xpm):
     return gtk.gdk.pixmap_colormap_create_from_xpm_d(None,
                                                      widget.get_colormap(),
@@ -151,8 +163,7 @@ def main():
     global trashcan_closed, trashcan_closed_mask
     global drag_icon, drag_mask
     win = gtk.Window()
-    if __name__ == '__main__':
-	win.connect('destroy', lambda win: gtk.main_quit())
+    win.connect('destroy', lambda win: gtk.main_quit())
     table = gtk.Table(2,2)
     win.add(table)
     drag_icon, drag_mask = create_pixmap(win, drag_icon_xpm)
@@ -171,13 +182,14 @@ def main():
     label.connect('drag_motion', popsite_motion)
     label.connect('drag_leave', popsite_leave)
 
-    pixmap = gtk.Pixmap(trashcan_closed, trashcan_closed_mask)
-    pixmap.drag_dest_set(0, [], 0)
-    table.attach(pixmap, 1, 2, 0, 1)
-    pixmap.connect('drag_leave', target_drag_leave)
-    pixmap.connect('drag_motion', target_drag_motion)
-    pixmap.connect('drag_drop', target_drag_drop)
-    pixmap.connect('drag_data_received', target_drag_data_received)
+    image = gtk.Image()
+    image.set_from_pixmap(trashcan_closed, trashcan_closed_mask)
+    image.drag_dest_set(0, [], 0)
+    table.attach(image, 1, 2, 0, 1)
+    image.connect('drag_leave', target_drag_leave)
+    image.connect('drag_motion', target_drag_motion)
+    image.connect('drag_drop', target_drag_drop)
+    image.connect('drag_data_received', target_drag_data_received)
 
     b = gtk.Button('Drag Here\n')
     b.drag_source_set(gtk.gdk.BUTTON1_MASK | gtk.gdk.BUTTON3_MASK,
@@ -187,6 +199,7 @@ def main():
     b.connect('drag_data_get', source_drag_data_get)
     b.connect('drag_data_delete', source_drag_data_delete)
     win.show_all()
-    if __name__ == '__main__': gtk.main()
+    gtk.main()
     
-if __name__ == '__main__': main()
+if __name__ == '__main__':
+    main()
