@@ -24,16 +24,6 @@ extern GType PY_TYPE_OBJECT;
 
 void  pyg_destroy_notify (gpointer user_data);
 
-typedef struct {
-    PyObject_HEAD
-    GParamSpec *pspec;
-} PyGParamSpec;
-extern PyTypeObject PyGParamSpec_Type;
-PyObject *pyg_param_spec_new(GParamSpec *pspec);
-
-#define PyGParamSpec_Check(v) (PyObject_TypeCheck(v, &PyGParamSpec_Type))
-#define PyGParamSpec_Get(v) (((PyGParamSpec *)v)->pspec)
-
 typedef struct {
     PyObject_HEAD
     GMainLoop *loop;
@@ -106,4 +96,7 @@ void       pyg_register_pointer (PyObject *dict, const gchar *class_name,
 				 GType pointer_type, PyTypeObject *type);
 PyObject * pyg_pointer_new      (GType pointer_type, gpointer pointer);
 
+extern PyTypeObject PyGParamSpec_Type;
+PyObject *pyg_param_spec_new    (GParamSpec *pspec);
+
 #endif
