@@ -521,6 +521,11 @@ pygobject_get_property(PyGObject *self, PyObject *args)
 			"the object does not support the given parameter");
 	return NULL;
     }
+    if (!(pspec->flags & G_PARAM_READABLE)) {
+	PyErr_Format(PyExc_TypeError, "property %s is not readable",
+		     param_name);
+	return NULL;
+    }
     g_value_init(&value, G_PARAM_SPEC_VALUE_TYPE(pspec));
     g_object_get_property(self->obj, param_name, &value);
     ret = pyg_value_as_pyobject(&value, TRUE);
@@ -546,6 +551,11 @@ pygobject_set_property(PyGObject *self, PyObject *args)
 			"the object does not support the given parameter");
 	return NULL;
     }
+    if (!(pspec->flags & G_PARAM_WRITABLE)) {
+	PyErr_Format(PyExc_TypeError, "property %s is not writable",
+		     param_name);
+	return NULL;
+    }
     g_value_init(&value, G_PARAM_SPEC_VALUE_TYPE(pspec));
     if (pyg_value_from_pyobject(&value, pvalue) < 0) {
 	PyErr_SetString(PyExc_TypeError,
@@ -948,7 +958,7 @@ pygobject_chain_from_overridden(PyGObject *self, PyObject *args)
 
     ihint = g_signal_get_invocation_hint(self->obj);
     if (!ihint) {
-	PyErr_SetString(PyExc_TypeError, "could not find signa invocation "
+	PyErr_SetString(PyExc_TypeError, "could not find signal invocation "
 			"information for this object.");
 	return NULL;
     }
@@ -981,7 +991,10 @@ pygobject_chain_from_overridden(PyGObject *self, PyObject *args)
     for (i = 0; i < query.n_params; i++) {
 	PyObject *item = PyTuple_GetItem(args, i);
 
-	if (pyg_value_from_pyobject(&params[i+1], item) < 0) {
+	if (pyg_boxed_check(item, (query.param_types[i] & ~G_SIGNAL_TYPE_STATIC_SCOPE))) {
+	    g_value_set_static_boxed(&params[i+1], pyg_boxed_get(item, void));
+	}
+	else if (pyg_value_from_pyobject(&params[i+1], item) < 0) {
 	    gchar buf[128];
 
 	    g_snprintf(buf, sizeof(buf),
@@ -1001,7 +1014,7 @@ pygobject_chain_from_overridden(PyGObject *self, PyObject *args)
     for (i = 0; i < query.n_params + 1; i++)
 	g_value_unset(&params[i]);
     g_free(params);
-    if ((query.return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE) != G_TYPE_NONE) {
+    if (query.return_type != G_TYPE_NONE) {
 	py_ret = pyg_value_as_pyobject(&ret, TRUE);
 	g_value_unset(&ret);
     } else {
